from .pypipr import *


""""""

Pypipr.test_print()

""""""

array = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]
print([i for i in sets_ordered(array)])

""""""

array = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]
print([i for i in list_unique(array)])

""""""

array = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]
print([i for i in chunck_array(array, 5)])

""""""

print_colorize("Print Colorize")

""""""


@log("Percobaan print log decorator")
def contoh_fungsi():
    pass


contoh_fungsi()

""""""

print_log("Percobaan print log standalone")

""""""

console_run("ls")
console_run("dir")

""""""

input_char("Input Char tanpa ditampilkan : ")

""""""

input_char_echo("Input Char dengan ditampilkan : ")

""""""

print(f"Is Windows : {WINDOWS}")

""""""

print(f"Is Linux : {LINUX}")

""""""

d = datetime_now()
print(f"Time now                : {d}")
d_jakarta = datetime_now("Asia/Jakarta")
print(f"Timezone Asia/Jakarta   : {d_jakarta}")
d_gmt = datetime_now("GMT")
print(f"Timezone GMT            : {d_gmt}")
d_utc = datetime_now("UTC")
print(f"Timezone UTC            : {d_utc}")
d_universal = datetime_now("Universal")
print(f"Timezone Universal      : {d_universal}")
d_gmt7 = datetime_now("Etc/GMT+7")
print(f"Timezone Etc/GMT+7      : {d_gmt7}")

""""""

file_put_contents("ifile_test.txt", "Contoh menulis content")

""""""

print(file_get_contents("ifile_test.txt"))

""""""

create_folder("contoh_membuat_folder")
create_folder("contoh/membuat/folder/recursive")
create_folder("./contoh_membuat_folder/secara/recursive")

""""""

for i in iscandir(recursive=False):
    print(i)

""""""

for i in scan_folder():
    print(i)

""""""

for i in scan_file():
    print(i)

""""""

regex_replacement_list = [
    {"regex": r"\{\{\s*(ini)\s*\}\}", "replacement": r"itu dan \1"},
    {"regex": r"\{\{\s*sini\s*\}\}", "replacement": "situ"},
]
data = "{{ ini }} adalah ini. {{sini}} berarti kesini."
data = regex_multiple_replace(data, regex_replacement_list, re.IGNORECASE)
print(data)

""""""

# print(html_get_contents("https://google.com/"))
print(r := url_get_contents("https://google.com/"))
assert r != False

""""""

a = html_get_contents("https://animekompi.net/", xpath="//a")
for i in a:
    print(f"{i.text} : {i.attrib}")

""""""

a = html_get_contents(
    "https://animekompi.net/", regex=r"(<a.[^>]+>(?:(?:\s+)?(.[^<]+)(?:\s+)?)<\/a>)"
)
for i in a:
    print(i)

""""""

a = html_get_contents("https://animekompi.net/", css_select="a")
for i in a:
    print(f"{i.text} : {i.attrib}")

""""""

print(get_filesize(__file__))

""""""

print(get_filemtime(__file__))

""""""

d = {
    "key1": "value1",
    "key2": "value2",
    "key3": "value3",
}
print(dict_first(d))

""""""

print(random_bool())

""""""


def gen(x):
    return x * 10


def pertama(result, **kwargs):
    print("pertama")
    result["pop"] = gen(kwargs["number"])


def kedua(result, **kwargs):
    print("kedua")
    result["lo"] = gen(kwargs["text"])


if __name__ == "__main__":
    # Tidak bisa dijalankan kalau dari import module, harus dari main program
    x = run_multiprocessing(
        {
            pertama: {"number": 55},
            kedua: {"text": "po"},
        }
    )
    for i in x:
        print(i)

""""""

set_timeout(1, lambda: print("Timeout 1"))
x = set_timeout(5, lambda: print("Timeout 5"))
wait(3)
print("wait 3")
print(x)
print("menghentikan timeout 5")
x.cancel()

""""""

print(datetime_from_string("2022-12-12 15:40:13").isoformat())

""""""


class alk:
    def a():
        return [x for x in range(10)]

    def b():
        return [x for x in range(10)]

    def c():
        return [x for x in range(10)]

    def d():
        return [x for x in range(10)]


print(compare_performance(get_class_method(alk)))



# Kode untuk mendapatkan semua list timezones
# import zoneinfo
# zoneinfo.available_timezones()
zoneinfo_available_timezones = [
    ["Africa", "Abidjan"],
    ["Africa", "Accra"],
    ["Africa", "Addis_Ababa"],
    ["Africa", "Algiers"],
    ["Africa", "Asmara"],
    ["Africa", "Asmera"],
    ["Africa", "Bamako"],
    ["Africa", "Bangui"],
    ["Africa", "Banjul"],
    ["Africa", "Bissau"],
    ["Africa", "Blantyre"],
    ["Africa", "Brazzaville"],
    ["Africa", "Bujumbura"],
    ["Africa", "Cairo"],
    ["Africa", "Casablanca"],
    ["Africa", "Ceuta"],
    ["Africa", "Conakry"],
    ["Africa", "Dakar"],
    ["Africa", "Dar_es_Salaam"],
    ["Africa", "Djibouti"],
    ["Africa", "Douala"],
    ["Africa", "El_Aaiun"],
    ["Africa", "Freetown"],
    ["Africa", "Gaborone"],
    ["Africa", "Harare"],
    ["Africa", "Johannesburg"],
    ["Africa", "Juba"],
    ["Africa", "Kampala"],
    ["Africa", "Khartoum"],
    ["Africa", "Kigali"],
    ["Africa", "Kinshasa"],
    ["Africa", "Lagos"],
    ["Africa", "Libreville"],
    ["Africa", "Lome"],
    ["Africa", "Luanda"],
    ["Africa", "Lubumbashi"],
    ["Africa", "Lusaka"],
    ["Africa", "Malabo"],
    ["Africa", "Maputo"],
    ["Africa", "Maseru"],
    ["Africa", "Mbabane"],
    ["Africa", "Mogadishu"],
    ["Africa", "Monrovia"],
    ["Africa", "Nairobi"],
    ["Africa", "Ndjamena"],
    ["Africa", "Niamey"],
    ["Africa", "Nouakchott"],
    ["Africa", "Ouagadougou"],
    ["Africa", "Porto-Novo"],
    ["Africa", "Sao_Tome"],
    ["Africa", "Timbuktu"],
    ["Africa", "Tripoli"],
    ["Africa", "Tunis"],
    ["Africa", "Windhoek"],
    ["America", "Adak"],
    ["America", "Anchorage"],
    ["America", "Anguilla"],
    ["America", "Antigua"],
    ["America", "Araguaina"],
    ["America", "Argentina", "Buenos_Aires"],
    ["America", "Argentina", "Catamarca"],
    ["America", "Argentina", "ComodRivadavia"],
    ["America", "Argentina", "Cordoba"],
    ["America", "Argentina", "Jujuy"],
    ["America", "Argentina", "La_Rioja"],
    ["America", "Argentina", "Mendoza"],
    ["America", "Argentina", "Rio_Gallegos"],
    ["America", "Argentina", "Salta"],
    ["America", "Argentina", "San_Juan"],
    ["America", "Argentina", "San_Luis"],
    ["America", "Argentina", "Tucuman"],
    ["America", "Argentina", "Ushuaia"],
    ["America", "Aruba"],
    ["America", "Asuncion"],
    ["America", "Atikokan"],
    ["America", "Atka"],
    ["America", "Bahia"],
    ["America", "Bahia_Banderas"],
    ["America", "Barbados"],
    ["America", "Belem"],
    ["America", "Belize"],
    ["America", "Blanc-Sablon"],
    ["America", "Boa_Vista"],
    ["America", "Bogota"],
    ["America", "Boise"],
    ["America", "Buenos_Aires"],
    ["America", "Cambridge_Bay"],
    ["America", "Campo_Grande"],
    ["America", "Cancun"],
    ["America", "Caracas"],
    ["America", "Catamarca"],
    ["America", "Cayenne"],
    ["America", "Cayman"],
    ["America", "Chicago"],
    ["America", "Chihuahua"],
    ["America", "Coral_Harbour"],
    ["America", "Cordoba"],
    ["America", "Costa_Rica"],
    ["America", "Creston"],
    ["America", "Cuiaba"],
    ["America", "Curacao"],
    ["America", "Danmarkshavn"],
    ["America", "Dawson"],
    ["America", "Dawson_Creek"],
    ["America", "Denver"],
    ["America", "Detroit"],
    ["America", "Dominica"],
    ["America", "Edmonton"],
    ["America", "Eirunepe"],
    ["America", "El_Salvador"],
    ["America", "Ensenada"],
    ["America", "Fort_Nelson"],
    ["America", "Fort_Wayne"],
    ["America", "Fortaleza"],
    ["America", "Glace_Bay"],
    ["America", "Godthab"],
    ["America", "Goose_Bay"],
    ["America", "Grand_Turk"],
    ["America", "Grenada"],
    ["America", "Guadeloupe"],
    ["America", "Guatemala"],
    ["America", "Guayaquil"],
    ["America", "Guyana"],
    ["America", "Halifax"],
    ["America", "Havana"],
    ["America", "Hermosillo"],
    ["America", "Indiana", "Indianapolis"],
    ["America", "Indiana", "Knox"],
    ["America", "Indiana", "Marengo"],
    ["America", "Indiana", "Petersburg"],
    ["America", "Indiana", "Tell_City"],
    ["America", "Indiana", "Vevay"],
    ["America", "Indiana", "Vincennes"],
    ["America", "Indiana", "Winamac"],
    ["America", "Indianapolis"],
    ["America", "Inuvik"],
    ["America", "Iqaluit"],
    ["America", "Jamaica"],
    ["America", "Jujuy"],
    ["America", "Juneau"],
    ["America", "Kentucky", "Louisville"],
    ["America", "Kentucky", "Monticello"],
    ["America", "Knox_IN"],
    ["America", "Kralendijk"],
    ["America", "La_Paz"],
    ["America", "Lima"],
    ["America", "Los_Angeles"],
    ["America", "Louisville"],
    ["America", "Lower_Princes"],
    ["America", "Maceio"],
    ["America", "Managua"],
    ["America", "Manaus"],
    ["America", "Marigot"],
    ["America", "Martinique"],
    ["America", "Matamoros"],
    ["America", "Mazatlan"],
    ["America", "Mendoza"],
    ["America", "Menominee"],
    ["America", "Merida"],
    ["America", "Metlakatla"],
    ["America", "Mexico_City"],
    ["America", "Miquelon"],
    ["America", "Moncton"],
    ["America", "Monterrey"],
    ["America", "Montevideo"],
    ["America", "Montreal"],
    ["America", "Montserrat"],
    ["America", "Nassau"],
    ["America", "New_York"],
    ["America", "Nipigon"],
    ["America", "Nome"],
    ["America", "Noronha"],
    ["America", "North_Dakota", "Beulah"],
    ["America", "North_Dakota", "Center"],
    ["America", "North_Dakota", "New_Salem"],
    ["America", "Nuuk"],
    ["America", "Ojinaga"],
    ["America", "Panama"],
    ["America", "Pangnirtung"],
    ["America", "Paramaribo"],
    ["America", "Phoenix"],
    ["America", "Port-au-Prince"],
    ["America", "Port_of_Spain"],
    ["America", "Porto_Acre"],
    ["America", "Porto_Velho"],
    ["America", "Puerto_Rico"],
    ["America", "Punta_Arenas"],
    ["America", "Rainy_River"],
    ["America", "Rankin_Inlet"],
    ["America", "Recife"],
    ["America", "Regina"],
    ["America", "Resolute"],
    ["America", "Rio_Branco"],
    ["America", "Rosario"],
    ["America", "Santa_Isabel"],
    ["America", "Santarem"],
    ["America", "Santiago"],
    ["America", "Santo_Domingo"],
    ["America", "Sao_Paulo"],
    ["America", "Scoresbysund"],
    ["America", "Shiprock"],
    ["America", "Sitka"],
    ["America", "St_Barthelemy"],
    ["America", "St_Johns"],
    ["America", "St_Kitts"],
    ["America", "St_Lucia"],
    ["America", "St_Thomas"],
    ["America", "St_Vincent"],
    ["America", "Swift_Current"],
    ["America", "Tegucigalpa"],
    ["America", "Thule"],
    ["America", "Thunder_Bay"],
    ["America", "Tijuana"],
    ["America", "Toronto"],
    ["America", "Tortola"],
    ["America", "Vancouver"],
    ["America", "Virgin"],
    ["America", "Whitehorse"],
    ["America", "Winnipeg"],
    ["America", "Yakutat"],
    ["America", "Yellowknife"],
    ["Antarctica", "Casey"],
    ["Antarctica", "Davis"],
    ["Antarctica", "DumontDUrville"],
    ["Antarctica", "Macquarie"],
    ["Antarctica", "Mawson"],
    ["Antarctica", "McMurdo"],
    ["Antarctica", "Palmer"],
    ["Antarctica", "Rothera"],
    ["Antarctica", "South_Pole"],
    ["Antarctica", "Syowa"],
    ["Antarctica", "Troll"],
    ["Antarctica", "Vostok"],
    ["Arctic", "Longyearbyen"],
    ["Asia", "Aden"],
    ["Asia", "Almaty"],
    ["Asia", "Amman"],
    ["Asia", "Anadyr"],
    ["Asia", "Aqtau"],
    ["Asia", "Aqtobe"],
    ["Asia", "Ashgabat"],
    ["Asia", "Ashkhabad"],
    ["Asia", "Atyrau"],
    ["Asia", "Baghdad"],
    ["Asia", "Bahrain"],
    ["Asia", "Baku"],
    ["Asia", "Bangkok"],
    ["Asia", "Barnaul"],
    ["Asia", "Beirut"],
    ["Asia", "Bishkek"],
    ["Asia", "Brunei"],
    ["Asia", "Calcutta"],
    ["Asia", "Chita"],
    ["Asia", "Choibalsan"],
    ["Asia", "Chongqing"],
    ["Asia", "Chungking"],
    ["Asia", "Colombo"],
    ["Asia", "Dacca"],
    ["Asia", "Damascus"],
    ["Asia", "Dhaka"],
    ["Asia", "Dili"],
    ["Asia", "Dubai"],
    ["Asia", "Dushanbe"],
    ["Asia", "Famagusta"],
    ["Asia", "Gaza"],
    ["Asia", "Harbin"],
    ["Asia", "Hebron"],
    ["Asia", "Ho_Chi_Minh"],
    ["Asia", "Hong_Kong"],
    ["Asia", "Hovd"],
    ["Asia", "Irkutsk"],
    ["Asia", "Istanbul"],
    ["Asia", "Jakarta"],
    ["Asia", "Jayapura"],
    ["Asia", "Jerusalem"],
    ["Asia", "Kabul"],
    ["Asia", "Kamchatka"],
    ["Asia", "Karachi"],
    ["Asia", "Kashgar"],
    ["Asia", "Kathmandu"],
    ["Asia", "Katmandu"],
    ["Asia", "Khandyga"],
    ["Asia", "Kolkata"],
    ["Asia", "Krasnoyarsk"],
    ["Asia", "Kuala_Lumpur"],
    ["Asia", "Kuching"],
    ["Asia", "Kuwait"],
    ["Asia", "Macao"],
    ["Asia", "Macau"],
    ["Asia", "Magadan"],
    ["Asia", "Makassar"],
    ["Asia", "Manila"],
    ["Asia", "Muscat"],
    ["Asia", "Nicosia"],
    ["Asia", "Novokuznetsk"],
    ["Asia", "Novosibirsk"],
    ["Asia", "Omsk"],
    ["Asia", "Oral"],
    ["Asia", "Phnom_Penh"],
    ["Asia", "Pontianak"],
    ["Asia", "Pyongyang"],
    ["Asia", "Qatar"],
    ["Asia", "Qostanay"],
    ["Asia", "Qyzylorda"],
    ["Asia", "Rangoon"],
    ["Asia", "Riyadh"],
    ["Asia", "Saigon"],
    ["Asia", "Sakhalin"],
    ["Asia", "Samarkand"],
    ["Asia", "Seoul"],
    ["Asia", "Shanghai"],
    ["Asia", "Singapore"],
    ["Asia", "Srednekolymsk"],
    ["Asia", "Taipei"],
    ["Asia", "Tashkent"],
    ["Asia", "Tbilisi"],
    ["Asia", "Tehran"],
    ["Asia", "Tel_Aviv"],
    ["Asia", "Thimbu"],
    ["Asia", "Thimphu"],
    ["Asia", "Tokyo"],
    ["Asia", "Tomsk"],
    ["Asia", "Ujung_Pandang"],
    ["Asia", "Ulaanbaatar"],
    ["Asia", "Ulan_Bator"],
    ["Asia", "Urumqi"],
    ["Asia", "Ust-Nera"],
    ["Asia", "Vientiane"],
    ["Asia", "Vladivostok"],
    ["Asia", "Yakutsk"],
    ["Asia", "Yangon"],
    ["Asia", "Yekaterinburg"],
    ["Asia", "Yerevan"],
    ["Atlantic", "Azores"],
    ["Atlantic", "Bermuda"],
    ["Atlantic", "Canary"],
    ["Atlantic", "Cape_Verde"],
    ["Atlantic", "Faeroe"],
    ["Atlantic", "Faroe"],
    ["Atlantic", "Jan_Mayen"],
    ["Atlantic", "Madeira"],
    ["Atlantic", "Reykjavik"],
    ["Atlantic", "South_Georgia"],
    ["Atlantic", "St_Helena"],
    ["Atlantic", "Stanley"],
    ["Australia", "ACT"],
    ["Australia", "Adelaide"],
    ["Australia", "Brisbane"],
    ["Australia", "Broken_Hill"],
    ["Australia", "Canberra"],
    ["Australia", "Currie"],
    ["Australia", "Darwin"],
    ["Australia", "Eucla"],
    ["Australia", "Hobart"],
    ["Australia", "LHI"],
    ["Australia", "Lindeman"],
    ["Australia", "Lord_Howe"],
    ["Australia", "Melbourne"],
    ["Australia", "NSW"],
    ["Australia", "North"],
    ["Australia", "Perth"],
    ["Australia", "Queensland"],
    ["Australia", "South"],
    ["Australia", "Sydney"],
    ["Australia", "Tasmania"],
    ["Australia", "Victoria"],
    ["Australia", "West"],
    ["Australia", "Yancowinna"],
    ["Brazil", "Acre"],
    ["Brazil", "DeNoronha"],
    ["Brazil", "East"],
    ["Brazil", "West"],
    ["CET"],
    ["CST6CDT"],
    ["Canada", "Atlantic"],
    ["Canada", "Central"],
    ["Canada", "Eastern"],
    ["Canada", "Mountain"],
    ["Canada", "Newfoundland"],
    ["Canada", "Pacific"],
    ["Canada", "Saskatchewan"],
    ["Canada", "Yukon"],
    ["Chile", "Continental"],
    ["Chile", "EasterIsland"],
    ["Cuba"],
    ["EET"],
    ["EST"],
    ["EST5EDT"],
    ["Egypt"],
    ["Eire"],
    ["Etc", "GMT"],
    ["Etc", "GMT+0"],
    ["Etc", "GMT+1"],
    ["Etc", "GMT+10"],
    ["Etc", "GMT+11"],
    ["Etc", "GMT+12"],
    ["Etc", "GMT+2"],
    ["Etc", "GMT+3"],
    ["Etc", "GMT+4"],
    ["Etc", "GMT+5"],
    ["Etc", "GMT+6"],
    ["Etc", "GMT+7"],
    ["Etc", "GMT+8"],
    ["Etc", "GMT+9"],
    ["Etc", "GMT-0"],
    ["Etc", "GMT-1"],
    ["Etc", "GMT-10"],
    ["Etc", "GMT-11"],
    ["Etc", "GMT-12"],
    ["Etc", "GMT-13"],
    ["Etc", "GMT-14"],
    ["Etc", "GMT-2"],
    ["Etc", "GMT-3"],
    ["Etc", "GMT-4"],
    ["Etc", "GMT-5"],
    ["Etc", "GMT-6"],
    ["Etc", "GMT-7"],
    ["Etc", "GMT-8"],
    ["Etc", "GMT-9"],
    ["Etc", "GMT0"],
    ["Etc", "Greenwich"],
    ["Etc", "UCT"],
    ["Etc", "UTC"],
    ["Etc", "Universal"],
    ["Etc", "Zulu"],
    ["Europe", "Amsterdam"],
    ["Europe", "Andorra"],
    ["Europe", "Astrakhan"],
    ["Europe", "Athens"],
    ["Europe", "Belfast"],
    ["Europe", "Belgrade"],
    ["Europe", "Berlin"],
    ["Europe", "Bratislava"],
    ["Europe", "Brussels"],
    ["Europe", "Bucharest"],
    ["Europe", "Budapest"],
    ["Europe", "Busingen"],
    ["Europe", "Chisinau"],
    ["Europe", "Copenhagen"],
    ["Europe", "Dublin"],
    ["Europe", "Gibraltar"],
    ["Europe", "Guernsey"],
    ["Europe", "Helsinki"],
    ["Europe", "Isle_of_Man"],
    ["Europe", "Istanbul"],
    ["Europe", "Jersey"],
    ["Europe", "Kaliningrad"],
    ["Europe", "Kiev"],
    ["Europe", "Kirov"],
    ["Europe", "Lisbon"],
    ["Europe", "Ljubljana"],
    ["Europe", "London"],
    ["Europe", "Luxembourg"],
    ["Europe", "Madrid"],
    ["Europe", "Malta"],
    ["Europe", "Mariehamn"],
    ["Europe", "Minsk"],
    ["Europe", "Monaco"],
    ["Europe", "Moscow"],
    ["Europe", "Nicosia"],
    ["Europe", "Oslo"],
    ["Europe", "Paris"],
    ["Europe", "Podgorica"],
    ["Europe", "Prague"],
    ["Europe", "Riga"],
    ["Europe", "Rome"],
    ["Europe", "Samara"],
    ["Europe", "San_Marino"],
    ["Europe", "Sarajevo"],
    ["Europe", "Saratov"],
    ["Europe", "Simferopol"],
    ["Europe", "Skopje"],
    ["Europe", "Sofia"],
    ["Europe", "Stockholm"],
    ["Europe", "Tallinn"],
    ["Europe", "Tirane"],
    ["Europe", "Tiraspol"],
    ["Europe", "Ulyanovsk"],
    ["Europe", "Uzhgorod"],
    ["Europe", "Vaduz"],
    ["Europe", "Vatican"],
    ["Europe", "Vienna"],
    ["Europe", "Vilnius"],
    ["Europe", "Volgograd"],
    ["Europe", "Warsaw"],
    ["Europe", "Zagreb"],
    ["Europe", "Zaporozhye"],
    ["Europe", "Zurich"],
    ["Factory"],
    ["GB"],
    ["GB-Eire"],
    ["GMT"],
    ["GMT+0"],
    ["GMT-0"],
    ["GMT0"],
    ["Greenwich"],
    ["HST"],
    ["Hongkong"],
    ["Iceland"],
    ["Indian", "Antananarivo"],
    ["Indian", "Chagos"],
    ["Indian", "Christmas"],
    ["Indian", "Cocos"],
    ["Indian", "Comoro"],
    ["Indian", "Kerguelen"],
    ["Indian", "Mahe"],
    ["Indian", "Maldives"],
    ["Indian", "Mauritius"],
    ["Indian", "Mayotte"],
    ["Indian", "Reunion"],
    ["Iran"],
    ["Israel"],
    ["Jamaica"],
    ["Japan"],
    ["Kwajalein"],
    ["Libya"],
    ["MET"],
    ["MST"],
    ["MST7MDT"],
    ["Mexico", "BajaNorte"],
    ["Mexico", "BajaSur"],
    ["Mexico", "General"],
    ["NZ"],
    ["NZ-CHAT"],
    ["Navajo"],
    ["PRC"],
    ["PST8PDT"],
    ["Pacific", "Apia"],
    ["Pacific", "Auckland"],
    ["Pacific", "Bougainville"],
    ["Pacific", "Chatham"],
    ["Pacific", "Chuuk"],
    ["Pacific", "Easter"],
    ["Pacific", "Efate"],
    ["Pacific", "Enderbury"],
    ["Pacific", "Fakaofo"],
    ["Pacific", "Fiji"],
    ["Pacific", "Funafuti"],
    ["Pacific", "Galapagos"],
    ["Pacific", "Gambier"],
    ["Pacific", "Guadalcanal"],
    ["Pacific", "Guam"],
    ["Pacific", "Honolulu"],
    ["Pacific", "Johnston"],
    ["Pacific", "Kanton"],
    ["Pacific", "Kiritimati"],
    ["Pacific", "Kosrae"],
    ["Pacific", "Kwajalein"],
    ["Pacific", "Majuro"],
    ["Pacific", "Marquesas"],
    ["Pacific", "Midway"],
    ["Pacific", "Nauru"],
    ["Pacific", "Niue"],
    ["Pacific", "Norfolk"],
    ["Pacific", "Noumea"],
    ["Pacific", "Pago_Pago"],
    ["Pacific", "Palau"],
    ["Pacific", "Pitcairn"],
    ["Pacific", "Pohnpei"],
    ["Pacific", "Ponape"],
    ["Pacific", "Port_Moresby"],
    ["Pacific", "Rarotonga"],
    ["Pacific", "Saipan"],
    ["Pacific", "Samoa"],
    ["Pacific", "Tahiti"],
    ["Pacific", "Tarawa"],
    ["Pacific", "Tongatapu"],
    ["Pacific", "Truk"],
    ["Pacific", "Wake"],
    ["Pacific", "Wallis"],
    ["Pacific", "Yap"],
    ["Poland"],
    ["Portugal"],
    ["ROC"],
    ["ROK"],
    ["Singapore"],
    ["Turkey"],
    ["UCT"],
    ["US", "Alaska"],
    ["US", "Aleutian"],
    ["US", "Arizona"],
    ["US", "Central"],
    ["US", "East-Indiana"],
    ["US", "Eastern"],
    ["US", "Hawaii"],
    ["US", "Indiana-Starke"],
    ["US", "Michigan"],
    ["US", "Mountain"],
    ["US", "Pacific"],
    ["US", "Samoa"],
    ["UTC"],
    ["Universal"],
    ["W-SU"],
    ["WET"],
    ["Zulu"],
]


months = [
    "january",
    "february",
    "march",
    "april",
    "may",
    "june",
    "july",
    "august",
    "september",
    "october",
    "november",
    "december",
]

weekday = [
    "sunday",
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday",
    "saturday",
]



""" PYPIPR Module """
from . import iconstant

"""PYTHON Standard Module"""
import datetime
import zoneinfo
import re
import subprocess
import platform
import pathlib
import urllib.request
import random
import webbrowser
import json
import shutil
import uuid
import time
import threading
import multiprocessing
import os
import timeit
import operator

# import math
# import asyncio

_platform_system = platform.system()
WINDOWS = _platform_system == "Windows"
LINUX = _platform_system == "Linux"

if WINDOWS:
    import msvcrt as _getch


"""PYPI Module"""
import colorama
import lxml.html

if LINUX:
    import getch as _getch


colorama.init()


class Pypipr:
    @staticmethod
    def test_print():
        """Print simple text to test this module is working"""
        print("Hello from PyPIPr")


def print_colorize(
    text,
    color=colorama.Fore.GREEN,
    bright=colorama.Style.BRIGHT,
    color_end=colorama.Style.RESET_ALL,
    text_start="",
    text_end="\n",
):
    """Print text dengan warna untuk menunjukan text penting"""
    print(f"{text_start}{color + bright}{text}{color_end}", end=text_end, flush=True)


def log(text):
    """
    Melakukan print ke console untuk menginformasikan proses yg sedang berjalan didalam program.
    """

    def inner_log(func):
        def callable_func(*args, **kwargs):
            print_log(text)
            result = func(*args, **kwargs)
            return result

        return callable_func

    return inner_log


def print_log(text):
    print_colorize(f">>> {text}")


def console_run(command):
    """Menjalankan command seperti menjalankan command di Command Terminal"""
    return subprocess.run(command, shell=True)


def input_char(prompt=None, prompt_ending="", newline_after_input=True):
    """Meminta masukan satu huruf tanpa menekan Enter. Masukan tidak ditampilkan."""
    if prompt:
        print(prompt, end=prompt_ending, flush=True)
    g = _getch.getch()
    if newline_after_input:
        print()
    return g


def input_char_echo(prompt=None, prompt_ending="", newline_after_input=True):
    """Meminta masukan satu huruf tanpa menekan Enter. Masukan akan ditampilkan."""
    if prompt:
        print(prompt, end=prompt_ending, flush=True)
    g = _getch.getche()
    if newline_after_input:
        print()
    return g


def datetime_now(timezone=None):
    """
    Datetime pada timezone tertentu
    """
    if timezone:
        return datetime.datetime.now(zoneinfo.ZoneInfo(timezone))
    else:
        return datetime.datetime.now()


def sets_ordered(iterator):
    """
    Hanya mengambil nilai unik dari suatu list
    """
    return tuple({i: {} for i in iterator}.keys())


def list_unique(iterator):
    """Sama seperti sets_ordered()"""
    return sets_ordered(iterator)


def chunck_array(array, size, start=0):
    """
    Membagi array menjadi potongan-potongan sebesar size
    """
    return tuple(array[i : i + size] for i in range(start, len(array), size))


def regex_multiple_replace(data, regex_replacement_list, flags=0):
    """
    Melakukan multiple replacement untuk setiap list.

    regex_replacement_list = [
        {"regex":r"", "replacement":""},
        {"regex":r"", "replacement":""},
        {"regex":r"", "replacement":""},
    ]
    """
    for v in regex_replacement_list:
        data = re.sub(v["regex"], v["replacement"], data, flags=flags)
    return data


def github_push(commit=None):
    def console(t, c):
        print_log(t)
        console_run(c)

    def console_input(prompt, default):
        print_colorize(prompt, text_end="")
        if default:
            print(default)
            return default
        else:
            return input()

    print_log("Menjalankan Github Push")
    console("Checking files", "git status")
    msg = console_input("Commit Message if any or empty to exit : ", commit)
    if msg:
        console("Mempersiapkan files", "git add .")
        console("Menyimpan files", f'git commit -m "{msg}"')
        console("Mengirim files", "git push")
    print_log("Selesai Menjalankan Github Push")


def github_pull():
    print_log("Git Pull")
    console_run("git pull")


def file_get_contents(filename):
    """
    Membaca seluruh isi file ke memory.
    Apabila file tidak ada maka akan return None.
    Apabila file ada tetapi kosong, maka akan return empty string
    """
    try:
        f = open(filename, "r")
        r = f.read()
        f.close()
        return r
    except:
        return None


def file_put_contents(filename, contents):
    """
    Menuliskan content ke file.
    Apabila file tidak ada maka file akan dibuat.
    Apabila file sudah memiliki content maka akan di overwrite.
    """
    f = open(filename, "w")
    r = f.write(contents)
    f.close()
    return r


def write_file(filename, contents):
    """
    Sama seperti file_put_contents()
    """
    return file_put_contents(filename, str(contents))


def read_file(filename):
    """
    Sama seperti file_get_contents()
    """
    return file_get_contents(filename)


def create_folder(folder_name):
    """
    Membuat folder.
    Membuat folder secara recursive dengan permission.
    """
    pathlib.Path(folder_name).mkdir(parents=True, exist_ok=True)


def iscandir(folder_name=".", glob_pattern="*", recursive=True):
    """
    Mempermudah scandir untuk mengumpulkan folder, subfolder dan file
    """
    if recursive:
        return pathlib.Path(folder_name).rglob(glob_pattern)
    else:
        return pathlib.Path(folder_name).glob(glob_pattern)


def scan_folder(folder_name="", glob_pattern="*", recursive=True):
    """
    Hanya mengumpulkan nama-nama folder dan subfolder.
    Tidak termasuk [".", ".."].
    """
    p = iscandir(
        folder_name=folder_name,
        glob_pattern=glob_pattern,
        recursive=recursive,
    )
    return tuple(i for i in p if i.is_dir())


def scan_file(folder_name="", glob_pattern="*", recursive=True):
    """
    Hanya mengumpulkan nama-nama file dalam folder dan subfolder.
    """
    p = iscandir(
        folder_name=folder_name,
        glob_pattern=glob_pattern,
        recursive=recursive,
    )
    return tuple(i for i in p if i.is_file())


def html_get_contents(url, xpath=None, regex=None, css_select=None):
    """
    Mengambil content html dari url.

    Return :
    - String            : Apabila hanya url saja yg diberikan
    - List of etree     : Apabila xpath diberikan
    - False             : Apabila terjadi error
    """
    url_req = urllib.request.Request(
        url=url,
        headers={
            "User-Agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Mobile Safari/537.36"
        },
    )
    url_open = urllib.request.urlopen(url_req)
    try:
        if xpath:
            return lxml.html.parse(url_open).findall(xpath)
        if regex:
            return re.findall(regex, url_open.read().decode())
        if css_select:
            return lxml.html.parse(url_open).getroot().cssselect(css_select)
        return url_open.read().decode()
    except:
        return False


def url_get_contents(url, xpath=None, regex=None, css_select=None):
    """
    Sama seperti html_get_contents()
    """
    return html_get_contents(url, xpath, regex, css_select)


def get_filesize(filename):
    """
    Mengambil informasi file size dalam bytes
    """
    return pathlib.Path(filename).stat().st_size


def get_filemtime(filename):
    """
    Mengambil informasi file size dalam bytes
    """
    return pathlib.Path(filename).stat().st_mtime_ns


def dict_first(d: dict) -> tuple:
    """
    Mengambil nilai (key, value) pertama dari dictionary dalam bentuk tuple
    """
    for k, v in d.items():
        return (k, v)


def random_bool() -> bool:
    """
    Menghasilkan nilai random antara 1 atau 0.
    fungsi ini merupkan fungsi tercepat untuk mendapatkan random bool value
    """
    return random.getrandbits(1)


def run_multiprocessing(func_dict: dict):
    """
    fungsi ini akan menjalankan fungsi secara bersamaan menggunakan module multiprocessing.
    Kemudian akan menunggu sampai semua selesai dijalankan.
    Kemudian akan mengembalikan nilai setiap fungsi berupa dictionary.
    Tidak bisa dijalankan kalau dari import module, Harus dari main program.
    """

    """
    contoh penggunaan:

    def gen(x):
        return x * 10

    def pertama(result, **kwargs):
        print("pertama")
        result["pop"] = gen(kwargs["number"])

    def kedua(result, **kwargs):
        print("kedua")
        result["lo"] = gen(kwargs["text"])

    if __name__ == "__main__":
        x = run_multiprocessing(
            {
                pertama: {"number": 55},
                kedua: {"text": "po"},
            }
        )
        for i in x:
            print(i)
    """

    r = []
    a = []
    m = multiprocessing.Manager()

    for i, v in func_dict.items():
        a.append(m.dict())
        r.append(multiprocessing.Process(target=i, args=(a[-1],), kwargs=v))

    for i in r:
        i.start()

    for i in r:
        i.join()

    return a


def set_timeout(interval, func, args=None, kwargs=None):
    """
    menjalankan fungsi ketika sudah sekian detik.
    apabila timeout masih berjalan tapi kode sudah selesai dieksekusi semua,
    maka program tidak akan berhenti sampai timeout selesai, kemudia fungsi dijalankan,
    kemudian program dihentikan.
    """
    t = threading.Timer(interval=interval, function=func, args=args, kwargs=kwargs)
    t.start()
    return t


def wait(seconds):
    """
    menunda kode program untuk menyelesaikan kode sebelumnya sampai beberapa waktu
    """
    time.sleep(seconds)


def datetime_from_string(iso_string, timezone=datetime.timezone.utc):
    """
    Parse iso string menjadi datetime object dengan timezone UTC
    """
    return datetime.datetime.fromisoformat(iso_string).replace(tzinfo=timezone)


def get_class_method(cls):
    """
    Mengembalikan berupa tuple yg berisi list dari method dalam class
    """
    return tuple(getattr(cls, x) for x in dir(cls) if not x.startswith("__"))


def compare_performance(func_list, n=100000, sort=True):
    """
    Menjalankan seluruh function dalam list, kemudian membandingkan waktu yg diperlukan.
    Hasil berupa tuple(nama_functions, perbandingan_waktu)
    perbandingan waktu berupa persentase, terkecil berarti tercepat.
    """
    x = tuple(timeit.timeit(i, number=n) for i in func_list)
    x_min = min(x)
    r = tuple((f.__name__, int(x[i] / x_min * 100)) for i, f in enumerate(func_list))
    if sort:
        return sorted(r, key=operator.itemgetter(1))
    else:
        return r



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypipr']

package_data = \
{'': ['*']}

install_requires = \
['colorama', 'cssselect', 'lxml', 'tzdata']

extras_require = \
{':platform_system == "Linux"': ['getch']}

setup_kwargs = {
    'name': 'pypipr',
    'version': '0.1.52',
    'description': 'The Python Package Index Project',
    'long_description': '# About\nThe Python Package Index Project (pypipr)\n\n\npypi : https://pypi.org/project/pypipr\n\n\n\n# Setup\nInstall with pip\n```\npython -m pip install pypipr\n```\n\nImport with * for fastest access\n```\nfrom pypipr.pypipr import *\n```\n\nTest with\n```python\nPypipr.test_print()\n```\n\n\n# Pypipr Class\n`test_print()` memastikan module sudah terinstal dan dapat dijalankan\n\n```python\nPypipr.test_print()\n```\n\n\n# functions\n`sets_ordered()` Hanya mengambil nilai unik dari suatu list\n\n```python\narray = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]\nprint([i for i in sets_ordered(array)])\n```\n\n\n`list_unique()` sama seperti `sets_ordered()`\n\n```python\narray = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]\nprint([i for i in list_unique(array)])\n```\n\n\n`chunck_array()` membagi array menjadi potongan dengan besaran yg diinginkan\n\n```python\narray = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]\nprint([i for i in chunck_array(array, 5)])\n```\n\n\n`print_colorize()` print ke console dengan warna\n\n```python\nprint_colorize("Print some text")\n```\n\n\n`@Log()` / `Log decorator` akan melakukan print ke console. Mempermudah pembuatan log karena tidak perlu mengubah fungsi yg sudah ada. Berguna untuk memberikan informasi proses program yg sedang berjalan.\n\n```python\n@log("Calling some function")\ndef some_function():\n    ...\n    return\n```\n\n\n`print_log` akan melakukan print ke console. Berguna untuk memberikan informasi proses program yg sedang berjalan.\n\n```python\nprint_log("Standalone Log")\n```\n\n\n`input_char()` meminta masukan satu huruf tanpa menekan enter. Char tidak ditampilkan.\n\n```python\ninput_char("Input Char without print : ")\n```\n\n\n`input_char()` meminta masukan satu huruf tanpa menekan enter. Char ditampilkan.\n\n```python\ninput_char_echo("Input Char n : ")\n```\n\n\n`datetime_now()` memudahkan dalam membuat tanggal dan waktu untuk suatu timezone\n\n```python\ndatetime_now("Asia/Jakarta")\ndatetime_now("GMT")\ndatetime_now("Etc/GMT+7")\n```\n\n\n`WINDOWS` True apabila berjalan di platform Windows\n\n```python\nprint(WINDOWS)\n```\n\n\n`LINUX` True apabila berjalan di platform Linux\n\n```python\nprint(LINUX)\n```\n\n\n`file_put_contents()` membuat file kemudian menuliskan contents ke file. Apabila file memiliki contents, maka contents akan di overwrite.\n\n```python\nfile_put_contents("ifile_test.txt", "Contoh menulis content")\n```\n\n\n`file_get_contents()` membaca contents file ke memory.\n\n```python\nprint(file_get_contents("ifile_test.txt"))\n```\n\n\n\n`write_file()` sama seperti `file_put_contents()`\n\n```python\nwrite_file("ifile_test.txt", "Contoh menulis content")\n```\n\n\n`read_file()` Sama seperti `file_get_contents()`\n\n```python\nprint(file_get_contents("ifile_test.txt"))\n```\n\n\n`create_folder()` membuat folder secara recursive.\n\n```python\ncreate_folder("contoh_membuat_folder")\ncreate_folder("contoh/membuat/folder/recursive")\ncreate_folder("./contoh_membuat_folder/secara/recursive")\n```\n\n\n`iscandir()` scan folder, subfolder, dan file\n\n```python\nfor i in iscandir():\n    print(i)\n```\n\n\n`scan_folder()` scan folder dan subfolder\n\n```python\nfor i in scan_folder():\n    print(i)\n```\n\n\n`scan_file()` scan file dalam folder dan subfolder\n\n```python\nfor i in scan_file():\n    print(i)\n```\n\n\n`regex_multiple_replace()` Melakukan multiple replacement untuk setiap list regex. \n\n```python\nregex_replacement_list = [\n    {"regex": r"\\{\\{\\s*(ini)\\s*\\}\\}", "replacement": r"itu dan \\1"},\n    {"regex": r"\\{\\{\\s*sini\\s*\\}\\}", "replacement": r"situ"},\n]\ndata = "{{ ini }} adalah ini. {{sini}} berarti kesini."\ndata = regex_multiple_replace(data, regex_replacement_list, re.IGNORECASE)\nprint(data)\n```\n\n\n`github_push()` simple github push dengan auto commit message\n\n```python\ngithub_push(\'Commit Message\')\n```\n\n\n`github_pull()` simple github pull\n\n```python\ngithub_pull()\n```\n\n\n`html_get_contents()` Mengambil content html dari url\n\n```python\n# Using XPATH\na = html_get_contents("https://google.com/", xpath="//a")\nfor i in a:\n    print(i.text)\n    print(i.attrib.get(\'href\'))\n\n# Using REGEX\na = html_get_contents("https://google.com/", regex=r"(<a.[^>]+>(?:(?:\\s+)?(.[^<]+)(?:\\s+)?)<\\/a>)")\nfor i in a:\n    print(i)\n\n# Using cssselect\na = html_get_contents("https://google.com/", css_select="a")\nfor i in a:\n    print(i.text)\n    print(i.attrib.get(\'href\'))\n\n```\n\n\n`get_filesize()` Mengambil informasi file size dalam bytes\n\n```python\nprint(get_filesize(__file__))\n```\n\n\n`get_filemtime()` Mengambil informasi last modification time file dalam nano seconds\n\n```python\nprint(get_filemtime(__file__))\n```\n\n\n`dict_first()` Mengambil nilai (key, value) pertama dari dictionary dalam bentuk tuple\n\n```python\nd = {\n    "key1": "value1",\n    "key2": "value2",\n    "key3": "value3",\n}\nprint(dict_first(d))\n```\n\n\n`random_bool()` Menghasilkan nilai random antara 1 atau 0\n\n```python\nprint(random_bool())\n```\n\n\n`run_multiprocessing()` Menjalankan program secara bersamaan\n```python\ndef gen(x):\n    return x * 10\n\ndef pertama(result, **kwargs):\n    print("pertama")\n    result["pop"] = gen(kwargs["number"])\n\ndef kedua(result, **kwargs):\n    print("kedua")\n    result["lo"] = gen(kwargs["text"])\n\nif __name__ == "__main__":\n    x = run_multiprocessing(\n        {\n            pertama: {"number": 55},\n            kedua: {"text": "po"},\n        }\n    )\n    for i in x:\n        print(i)\n```\n\n\n`set_timeout()` Menjalankan fungsi ketika sudah sekian detik.\n\n```python\nset_timeout(3, lambda: print("Timeout 3"))\nx = set_timeout(7, lambda: print("Timeout 7"))\nprint(x)\nprint("menghentikan timeout 7")\nx.cancel()\n```\n\n\n`wait()` Menunda kode program untuk menyelesaikan kode sebelumnya sampai beberapa waktu.\n\n```python\nwait(5)\nprint("wait 5")\n```\n\n\n`datetime_from_string()` Parse iso_string menjadi datetime object dengan timezone UTC\n\n```python\nprint(datetime_from_string("2022-12-12 15:40:13").isoformat())\n```\n\n\n`get_class_method()` Mengembalikan berupa tuple yg berisi list dari method dalam class\n\n```python\nclass alk:\n    def a():\n        return [x for x in range(10)]\n\n    def b():\n        return [x for x in range(10)]\n\n    def c():\n        return [x for x in range(10)]\n\n    def d():\n        return [x for x in range(10)]\n\n\nprint(get_class_method(alk))\n\n```\n\n\n`compare_performance()` Menjalankan seluruh function dalam list, kemudian membandingkan waktu yg diperlukan.\n\n```python\nclass alk:\n    def a():\n        return [x for x in range(10)]\n\n    def b():\n        return [x for x in range(10)]\n\n    def c():\n        return [x for x in range(10)]\n\n    def d():\n        return [x for x in range(10)]\n\n\nprint(compare_performance(get_class_method(alk)))\n\n```\n',
    'author': 'ufiapjj',
    'author_email': 'ufiapjj@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)
